
/*
'***************************************************************************
'Copyright (C) 2010-2011 KBasic Software: www.kbasic.com. All rights reserved.
'Source codes are created by Bernd Noetscher sales@kbasic.com 
'
'You may use this source codes under the terms of the GNU Public License (Version 3) as published 
'by the Free Software Foundation. 
'
'If you are interested in using the source codes for other licenses or 
'commercial development, you must buy commercial licenses from KBasic Software.
'***************************************************************************
*/
 









#include "Q7BCompiler.h"
#include <QtCore>


#ifdef Q7BGCC

#include <QtGui>
 #include <QMessageBox>

#endif

#ifdef Q_WS_MAC
#include <Carbon/Carbon.h>
#endif

#ifdef Q_OS_WIN32
  #include <windows.h> 
#endif

QString sMyError;


QString LoadFile2(QString sFile)
{
  QString s;
  QFile f(sFile);
  if (f.open(QIODevice::ReadOnly)){
    QTextStream t(&f);
    s = t.readAll();
    f.close();
  }
  return s;
}

void SaveFile2(QString sFile, QString s)
{
  QFile f(sFile);
  if (f.open(QIODevice::WriteOnly | QIODevice::Truncate)){
    QTextStream t(&f);
    t << s;
    f.close();
  }
}

QString Q7BCompiler::getName(QString s)
{
  int n = s.lastIndexOf("/");
  if (n == -1) n = s.lastIndexOf("\\");
  if (n != -1) s = s.mid(n + 1);
  n = s.lastIndexOf(".");
  if (n != -1) s = s.left(n);
  return s;
}

QString Q7BCompiler::Go2(const QString & sProjectPath, const QString & sProjectName, bool bDoRunForGo)
{
  

  QString sProjectPath2 = sProjectPath;
  QString sProjectName2 = getName(sProjectName);

  if (sProjectPath2.endsWith(".q7basic")){ 
    sProjectPath2 = QDir::tempPath() + "/Q7Basic/" + getName(sProjectPath2);



  }

#ifdef Q_OS_WIN32
  QString s = sProjectPath2 + "/gcc/Build/release/" + sProjectName2 + ".exe";
#elif defined Q_WS_MAC
  QString s = sProjectPath2 + "/gcc/Build/" + sProjectName2 + ".app/Contents/MacOS/" + sProjectName2;
#elif defined Q_WS_X11
  QString s = sProjectPath2 + "/gcc/Build/" + sProjectName2;
#endif

  if (QFile::exists(s)){

      

   

    #ifdef Q7BGCC
      emit ReadyForDebugging();
      qApp->processEvents();
    #endif  

    if (bDebug){   


  

      QFile::remove(QDir::tempPath() + "/Q7Basic/" + "q7b_out.txt");

      QString m = "breakpoints\n";
      QMapIterator<QString, QStringList> i(breakpoints);
      while (i.hasNext()) {
         i.next();
         if (i.value().count() > 0){
           m += QFileInfo(i.key()).fileName() + "\n";
           m += i.value().join(";") + "\n";
         }
      }

    
      SaveFile2(QDir::tempPath() + "/Q7Basic/" + "q7b_out.txt", m);      
    }

   
    bGo = true;
    this->p.start("\"" + s + "\""); 

    

    p.waitForStarted(-1);
    if (this->p.state() == QProcess::Running){
      while(this->p.pid() <= 0){}
    }


  qint64 PID;
#ifdef Q_OS_WIN32
    PID = p.pid()->dwProcessId;
#else
    PID = p.pid();
#endif



      QFile::remove(QDir::tempPath() + "/Q7Basic/" + "q7b_out" + QString("%1").arg(PID) + ".txt");
      QFile::remove(QDir::tempPath() + "/Q7Basic/" + "q7b_in" + QString("%1").arg(PID) + ".txt");
 

      QString sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" + QString("%1").arg(PID) + ".txt";
      SaveFile2(sDebugFile, "");  
      
 

 #ifdef Q7BGCC
#ifdef Q_WS_MAC
    if (this->p.state() == QProcess::Running){

       ProcessSerialNumber psn;
            OSStatus st;
            do {
              st = GetProcessForPID(this->p.pid(), &psn);
            } while (st == -600);

        
        
      if (st >=0 ){
      SetFrontProcess(&psn);
    }
  }
#endif
    #endif

    
    if (bDebug){

      QString sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_in" + QString("%1").arg(PID) + ".txt";
      SaveFile2(sDebugFile, "");
      watch.addPath(sDebugFile);   

      while (QFileInfo(sDebugFile).size() == 0){  
        qApp->processEvents(); 
      }
      fileChanged(sDebugFile); 

   }

  }
  return "";
}

void Q7BCompiler::fileChanged( const QString  & path)
{

  qint64 PID;
#ifdef Q_OS_WIN32
    PID = p.pid()->dwProcessId;
#else
    PID = p.pid();
#endif

  QString sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_in" + QString("%1").arg(PID) + ".txt";

  if (QFile::exists(sDebugFile)){ 
    QString s = LoadFile2(sDebugFile);
    if (!s.isEmpty()){
      

    #ifdef Q7BGCC

      

      if (s.startsWith("ClearBreakpoint\n")){
          emit BreakpointReached("", "");
      } else if (s.startsWith("Breakpoint\n")){
        QStringList l = s.split("\n");
        if (l.size() > 0){
          if (l.at(0) == "Breakpoint"){
            
            emit BreakpointReached(l.at(1).simplified(), l.at(2).simplified());
          }
        }
      }
    #endif  

    }
      
   }
   
}

QString Q7BCompiler::Go(const QString & sProjectPath, const QString & sProjectName)
{
  bRun = true;
  bDebug = false;
  bDeployment = false;
 
  _sProjectPath = sProjectPath;
  _sProjectName = sProjectName;

  return Go2(sProjectPath, sProjectName, true);
}

QString Q7BCompiler::Make(const QString & sProjectPath)
{
  bRun = false;
  bDebug = false;
  bMayRun = false;
  bDeployment = false;

#ifdef Q_OS_WIN32
  QString program = qApp->applicationDirPath() + "/Q7BCompiler.exe";
#else
  QString program = qApp->applicationDirPath() + "/Q7BCompiler";
#endif
  QStringList arguments;
  arguments << sProjectPath << "-Make" << "-VCDIR" + sVC << "-QTDIR" + sQt << "-MINGWDIR" + sMINGWDIR;  
  myclose();
  this->p.start(program, arguments);
  this->p.waitForStarted(-1);

  return "";
}

QString Q7BCompiler::Build(const QString & sProjectPath)
{
  bRun = false;
  bDebug = false;
  bMayRun = false;
  bDeployment = false;

#ifdef Q_OS_WIN32
  QString program = qApp->applicationDirPath() + "/Q7BCompiler.exe";
#else
  QString program = qApp->applicationDirPath() + "/Q7BCompiler";
#endif
  QStringList arguments;
  arguments << sProjectPath << "-Build" << "-VCDIR" + sVC << "-QTDIR" + sQt << "-MINGWDIR" + sMINGWDIR;
  myclose(); 
  this->p.start(program, arguments);
  this->p.waitForStarted(-1);
  
  return "";
}

QString Q7BCompiler::Deploy(const QString & sProjectPath)
{
  bRun = false;
  bDebug = false;
  bMayRun = false;
  bDeployment = true;

#ifdef Q_OS_WIN32
  QString program = qApp->applicationDirPath() + "/Q7BCompiler.exe";
#else
  QString program = qApp->applicationDirPath() + "/Q7BCompiler";
#endif
  QStringList arguments;
  arguments << sProjectPath << "-Deploy" << "-VCDIR" + sVC << "-QTDIR" + sQt << "-MINGWDIR" + sMINGWDIR;
  myclose();
  this->p.start(program, arguments);
  this->p.waitForStarted(-1);
  
  return "";
}

QString Q7BCompiler::Run(const QString & sProjectPath, const QString & sProjectName)
{
  bRun = true;
  bDebug = false;
  bDeployment = false;

  _sProjectPath = sProjectPath;
  _sProjectName = sProjectName;

#ifdef Q_OS_WIN32
  QString program = qApp->applicationDirPath() + "/Q7BCompiler.exe";
#else
  QString program = qApp->applicationDirPath() + "/Q7BCompiler";
#endif
  QStringList arguments;
  arguments << sProjectPath << "-Run" << "-VCDIR" + sVC << "-QTDIR" + sQt << "-MINGWDIR" + sMINGWDIR;
  myclose();
  this->p.start(program, arguments);
  this->p.waitForStarted(-1);



  return "";
}

QString Q7BCompiler::Debug(const QString & sProjectPath, const QString & sProjectName)
{
  bDeployment = false;

  if (bDebug && this->p.state() == QProcess::Running){
    Continue();
    return "";
  } else {

    bRun = true;
    

    _sProjectPath = sProjectPath;
    _sProjectName = sProjectName;

  #ifdef Q_OS_WIN32
    QString program = qApp->applicationDirPath() + "/Q7BCompiler.exe";
  #else
    QString program = qApp->applicationDirPath() + "/Q7BCompiler";
  #endif
    QStringList arguments;
    arguments << sProjectPath << "-Run" << "-VCDIR" + sVC << "-QTDIR" + sQt << "-MINGWDIR" + sMINGWDIR;
    myclose();
    bDebug = true;
    this->p.start(program, arguments);
    this->p.waitForStarted(-1);



    return "";
  }
}

QString Q7BCompiler::Stop(QString sProjectPath)
{

  bGo = false;
  if (bDebug){
    bDebug = false;
    SendStop();

    if (bRun){
      this->p.close();
    } else {
      myclose();
    }    

  } else {

    if (bRun){
      this->p.close();
    } else {
      myclose();
    }    

  }


  return "";
}

void Q7BCompiler::setVCDir(QString s)
{
  sVC = s;
}

void Q7BCompiler::setQtDir(QString s)
{
  sQt = s;
}

void Q7BCompiler::setMINGWDir(QString s)
{
  sMINGWDIR = s;

}

QString sStdOut;

void Q7BCompiler::finished ( int exitCode, QProcess::ExitStatus  exitStatus )  
{
  

  if (exitStatus == QProcess::CrashExit){
    
  } else {

    if (bRun && bMayRun){
      Go2(_sProjectPath, _sProjectName, false);
      bMayRun = false;
    } else {
   

      #ifdef Q7BGCC
        emit CompilerOrApplicationFinished(bGo);
      #endif  

    }
  }
  
}


void Q7BCompiler::readyReadStandardError()
{

  QString s = QString::fromLocal8Bit(p.readAllStandardError().constData());

    

  if (s.isEmpty()) return;

  if (s.contains("D9025")) return;  

  if (s.contains("syntax error or parser error in line ")){
    bQ7BCompilerNotCPP = true;
  } else {
    bQ7BCompilerNotCPP = false;
  }

  if (bGo){
#ifdef Q7BGCC
    sMyError = sError = s;
    emit StandardError(sMyError);  
#endif

    return;
  } else {
#ifdef Q7BGCC
    static QString rr; rr = s;
  
    emit StandardErrorOfCompiler(rr); 
#endif
  }
  

  

#ifdef Q7BGCC
  sMyError = sError = s;

  
  sMyError = (sStdOut.contains("error") ?  sStdOut : "") + sMyError;
  
  emit Error(sMyError);  
  bRun = false;
  bMayRun = false;
#endif

  

}

void Q7BCompiler::readyReadStandardOutput()
{

  QString s = QString::fromLocal8Bit(p.readAllStandardOutput().constData());

  
  
  if (s.contains(") : error")){ 

    sMyError = sError = s.mid(s.indexOf(") : error") + 4);

    bQ7BCompilerNotCPP = false;
 #ifdef Q7BGCC
    emit StandardErrorOfCompiler(sMyError);
  #endif  
    myclose();
    return;
  }
    

   

  if (s.isEmpty()) return;

  sStdOut = s;

   if (bGo){
#ifdef Q7BGCC
    emit StandardOutput(sStdOut);  
#endif

    return;
  } else {
#ifdef Q7BGCC
    static QString rr; 

    if (sStdOut.contains("OBJECTFILES:")){
      int n = sStdOut.indexOf("OBJECTFILES");
      int n2 = sStdOut.indexOf("\n", n);
      rr = sStdOut.left(n) + sStdOut.mid(n2);
      sStdOut = sStdOut.mid(n, n2 - n);
    } else {
      rr = sStdOut;
    }

    if (!rr.startsWith("OBJECTFILES:")){
      rr = rr.replace("Q7BFinished", "");      
      emit StandardOutputOfCompiler(rr);  
    }    
#endif
  }
  
  

  if (s.startsWith("OBJECTFILES:")){
 
     
    int n = s.indexOf("\n");
    QString k = s.mid(13, n - 13);
    s = s.mid(n + 1);
    sOBJECTFILES = k.split("||", QString::SkipEmptyParts);

    this->nCountDone = 0;
#ifdef Q7BGCC
    int kk = 0;
    if (sOBJECTFILES.count() > 0){
      kk = sOBJECTFILES.count(); 
    }
  emit Started(kk);  
#endif  
  }


 
  if (s.startsWith("Q7BFinished") || s.endsWith("Q7BFinished")){
    
#ifdef Q7BGCC
  emit Finished(bDeployment ? -999999 : this->nCountDone);
#endif  
  
  bMayRun = true;

    s = s.mid(12);
    if (s.isEmpty()) return;
    
  }
  bool b = false;

#ifdef Q_OS_WIN32
  b = sMINGWDIR.isEmpty();
#endif
  QStringList l;
  if (b){
    l = s.split(".cpp\r\n", QString::SkipEmptyParts);
  } else {
    l = s.split("\ng++", QString::SkipEmptyParts);
  }


  for (int i = 0; i < l.count(); i++){
    QString k = l.at(i);

  
    
    QString sFilename = k.trimmed();
  bool bb = false;

#ifdef Q_OS_WIN32
  bb = sMINGWDIR.isEmpty();
#endif

  if (bb){
    if (sFilename.contains(" ")) continue;
  } else {
    int m = k.lastIndexOf(" ");
    if (m == -1) m = k.lastIndexOf("\n");
    if (m == -1) continue;
    sFilename = k.mid(m + 1).trimmed();
    if (!sFilename.endsWith(".cpp")) continue;
    int k = sFilename.indexOf("release\\");
    if (k >= 0) sFilename = sFilename.mid(k + 8);
  }



 

    if (this->sOBJECTFILES.contains(sFilename)){
      this->nCountDone++;
      #ifdef Q7BGCC
        emit Compiled(sFilename.left(sFilename.length() - 4), this->nCountDone);
      #endif  
      int n = sOBJECTFILES.indexOf(sFilename);
      sOBJECTFILES.removeAt(n);
    } else {
      this->nCountDone++;
      #ifdef Q7BGCC
        emit Compiled(sFilename.left(sFilename.length() - 4), -1);
      #endif  
    }
  }

  
  if (s.contains(" -lQtGui") && s.contains(" -lQtCore") && s.contains(" -L\"")){
 #ifdef Q7BGCC
    emit Compiled("", -2);
  #endif  
  }
}

void Q7BCompiler::myclose()
{
  if (bQ7BCompilerNotCPP){

    this->p.close(); 

  } else {

    if (this->p.state() == QProcess::Running && bGo == false){

      QFile f(qApp->applicationDirPath() + "/stop.txt");
      if (!f.open(QIODevice::WriteOnly )){
      } else {
        QTextStream t(&f);
        

  #ifdef Q_OS_WIN32
    t << p.pid()->dwProcessId;
  #else
    t << p.pid();
  #endif

        f.close();   
      }


    }



    QDateTime dt = QDateTime::currentDateTime().addSecs(4);


    
    
    while (this->p.state() == QProcess::Running && QDateTime::currentDateTime() < dt){
      qApp->processEvents();
    }

     

     

    

    this->p.close();
  }
}

void Q7BCompiler::Continue()
{
  if (bDebug && this->p.state() == QProcess::Running){

    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif

    SaveFile2(sDebugFile, "Continue");
  }
}

void Q7BCompiler::StepInto()
{
  if (bDebug && this->p.state() == QProcess::Running){

    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif

    SaveFile2(sDebugFile, "StepInto");
  }
}

void Q7BCompiler::StepOut()
{
  if (bDebug && this->p.state() == QProcess::Running){
    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif


    SaveFile2(sDebugFile, "StepOut");
  }
}

void Q7BCompiler::StepOver()
{
  if (bDebug && this->p.state() == QProcess::Running){
    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif


    SaveFile2(sDebugFile, "StepOver");
  }
}

void Q7BCompiler::RunToCursor(QString sFile, QString sLine)
{
  if (bDebug && this->p.state() == QProcess::Running){
    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif


    SaveFile2(sDebugFile, "RunToCursor\n" + QFileInfo(sFile).fileName() + "\n" + sLine);
  }
}

void Q7BCompiler::Breakpoint(QString sFile, QString sLine)
{
  if (bDebug && this->p.state() == QProcess::Running){
    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "q7b_out" +  QString("%1").arg(p.pid()) + ".txt";
  #endif


    SaveFile2(sDebugFile, "Breakpoint\n" + QFileInfo(sFile).fileName() + "\n" + sLine);
  }
}

void Q7BCompiler::SendStop()
{
  if (bDebug && this->p.state() == QProcess::Running){

    QString sDebugFile;

  #ifdef Q_OS_WIN32
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "stop" +  QString("%1").arg(p.pid()->dwProcessId) + ".txt";
  #else
    sDebugFile = QDir::tempPath() + "/Q7Basic/" + "stop" +  QString("%1").arg(p.pid()) + ".txt";
  #endif

    SaveFile2(sDebugFile, "");
  }
}
